function varargout = controlGUI(varargin)
% CONTROLGUI M-file for controlGUI.fig
%      CONTROLGUI, by itself, creates a new CONTROLGUI or raises the existing
%      singleton*.
%
%      H = CONTROLGUI returns the handle to a new CONTROLGUI or the handle to
%      the existing singleton*.
%
%      CONTROLGUI('CALLBACK',hObject,eventData,handles,...) calls the local
%      function named CALLBACK in CONTROLGUI.M with the given input arguments.
%
%      CONTROLGUI('Property','Value',...) creates a new CONTROLGUI or raises the
%      existing singleton*.  Starting from the left, property value pairs are
%      applied to the GUI before controlGUI_OpeningFcn gets called.  An
%      unrecognized property name or invalid value makes property application
%      stop.  All inputs are passed to controlGUI_OpeningFcn via varargin.
%
%      *See GUI Options on GUIDE's Tools menu.  Choose "GUI allows only one
%      instance to run (singleton)".
%
% See also: GUIDE, GUIDATA, GUIHANDLES

% Edit the above text to modify the response to help controlGUI

% Last Modified by GUIDE v2.5 07-Jul-2010 11:37:41

% Begin initialization code - DO NOT EDIT
gui_Singleton = 1;
gui_State = struct('gui_Name',       mfilename, ...
                   'gui_Singleton',  gui_Singleton, ...
                   'gui_OpeningFcn', @controlGUI_OpeningFcn, ...
                   'gui_OutputFcn',  @controlGUI_OutputFcn, ...
                   'gui_LayoutFcn',  [] , ...
                   'gui_Callback',   []);
if nargin && ischar(varargin{1})
    gui_State.gui_Callback = str2func(varargin{1});
end

if nargout
    [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
else
    gui_mainfcn(gui_State, varargin{:});
end
% End initialization code - DO NOT EDIT


% --- Executes just before controlGUI is made visible.
function controlGUI_OpeningFcn(hObject, eventdata, handles, varargin)
% This function has no output args, see OutputFcn.
% hObject    handle to figure
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
% varargin   command line arguments to controlGUI (see VARARGIN)

clc;
% Close all active serial ports
if ~isempty(instrfind); fclose(instrfind); delete(instrfind); end

% Set info (if changed, also need to change on the PIC)
obj = serial('COM9','baudrate',38400);
obj.FlowControl = 'Hardware';
obj.InputBufferSize = 5000;
obj.OutputBufferSize = 20000;
handles.obj = obj;
fopen(handles.obj);

% Initialize vectors
handles.counts_per_rev = 100000;
handles.time = 2.5;
handles.dt = .001;
handles.NUM_PTS = handles.time/handles.dt;
handles.data = zeros(1,handles.NUM_PTS);
handles.traj = zeros(1,handles.NUM_PTS);

% Set control gains
set(handles.KP,'String',1000);
set(handles.KI,'String',20);
set(handles.KD,'String',10);

% Initialize trajectory
% trajectory = {['DT = ' num2str(handles.dt)],['N = ' num2str(handles.NUM_PTS)],...
%     'j = 1','k = -0.625','l = 0','T = 1',...
%     '','t = DT:DT:N*DT','for i = 1:2*T/DT',...
%     '   theta(i) = 90 + 90*(j*(t(i)-T) + k*(t(i)-T)^3 + l*(t(i)-T)^5)/(j*T + k*T^3 + l*T^5)','end',...
%     'for i = 2*T/DT+1:N','   theta(i) = 180','end'};
trajectory = {['DT = ' num2str(handles.dt)],['N = ' num2str(handles.NUM_PTS)],...
    '','t = DT:DT:N*DT','for i = 1:N',...
    '   theta(i) = 60*sin(2*pi*t(i))','end'};
set(handles.trajectory,'String',trajectory);

% Plot trajectory
trajectory_Callback(handles.trajectory,[],handles);

% Choose default command line output for controlGUI
handles.output = hObject;

% Update handles structure
guidata(hObject, handles);

% UIWAIT makes controlGUI wait for user response (see UIRESUME)
% uiwait(handles.figure1);


% --- Outputs from this function are returned to the command line.
function varargout = controlGUI_OutputFcn(hObject, eventdata, handles) 
% varargout  cell array for returning output args (see VARARGOUT);
% hObject    handle to figure
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Get default command line output from handles structure
varargout{1} = handles.output;


% --- Executes on button press in run.
function run_Callback(hObject, eventdata, handles)
% hObject    handle to run (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

set(handles.status,'String','Running'); pause(.001);
fwrite(handles.obj,4,'int32');

pause(handles.time);
set(handles.status,'String','Idle'); pause(.001);
% Update handles structure
guidata(hObject, handles);


% --- Executes on button press in collect.
function collect_Callback(hObject, eventdata, handles)
% hObject    handle to collect (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

traj = handles.traj;
data = handles.data;

% Collect trajectory info
set(handles.status,'String','Receiving Data'); pause(.001);
fwrite(handles.obj,2,'int32');
for i = 1:handles.NUM_PTS
    reading = fscanf(handles.obj,'%s\n');
    if numel(str2num(reading))~= 2
        break;
    end
    temp = str2num(reading)*360/handles.counts_per_rev;
    traj(i) = temp(1);
    data(i) = temp(2);
end

% Plot data
set(handles.status,'String','Plotting Data'); pause(.001);
t = handles.dt:handles.dt:handles.time;

plot(t,traj, t,data,'r');
xlabel('Time (s)'); ylabel('\theta (\circ)');

set(handles.status,'String','Idle');
% Update handles structure
handles.traj = traj;
handles.data = data;
guidata(hObject, handles);


% --- Executes on button press in sendK.
function sendK_Callback(hObject, eventdata, handles)
% hObject    handle to sendK (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

set(handles.status,'String','Sending Gains'); pause(.001);

str = {'KP','KI','KD'};
mat = ones(1,numel(str)+1);
for i = 2:numel(mat)
    mat(i) = eval(['str2double(get(handles.' str{i-1} ',''String''))']);
end
fwrite(handles.obj,mat,'int32')

set(handles.status,'String','Receiving Data'); pause(.001);
fwrite(handles.obj,3,'int32');
str=fscanf(handles.obj,'%s\n');
values = str2num(str);
for i = 1:numel(values)
    if values(i) ~= mat(i+1)
        eval(['set(handles.' str{i} ',''String'',' num2str(values(i)) ');']);
    end
end

set(handles.status,'String','Idle'); pause(.001);

% Update handles structure
guidata(hObject, handles);


% --- Executes on button press in sendTraj.
function sendTraj_Callback(hObject, eventdata, handles)
% hObject    handle to sendTraj (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
set(handles.status,'String','Sending Trajectory'); pause(.001);

mat = [5 handles.traj*handles.counts_per_rev/360];
fwrite(handles.obj,mat,'int32')

% fwrite(handles.obj,handles.PWM,'int32')

set(handles.status,'String','Idle'); pause(.001);

% Update handles structure
guidata(hObject, handles);



% --- Executes on button press in save.
function save_Callback(hObject, eventdata, handles)
% hObject    handle to save (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
assignin('base','handles',handles)


function KP_Callback(hObject, eventdata, handles)
% hObject    handle to KP (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of KP as text
%        str2double(get(hObject,'String')) returns contents of KP as a
%        double
temp = str2double(get(handles.KP,'String'));
set(handles.KP,'String',num2str(round(temp)));
% Update handles structure
guidata(hObject, handles);


% --- Executes during object creation, after setting all properties.
function KP_CreateFcn(hObject, eventdata, handles)
% hObject    handle to KP (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end



function KI_Callback(hObject, eventdata, handles)
% hObject    handle to KI (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of KI as text
%        str2double(get(hObject,'String')) returns contents of KI as a
%        double
temp = str2double(get(handles.KI,'String'));
set(handles.KI,'String',num2str(round(temp)));
% Update handles structure
guidata(hObject, handles);


% --- Executes during object creation, after setting all properties.
function KI_CreateFcn(hObject, eventdata, handles)
% hObject    handle to KI (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end



function KD_Callback(hObject, eventdata, handles)
% hObject    handle to KD (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of KD as text
%        str2double(get(hObject,'String')) returns contents of KD as a
%        double
temp = str2double(get(handles.KD,'String'));
set(handles.KD,'String',num2str(round(temp)));
% Update handles structure
guidata(hObject, handles);


% --- Executes during object creation, after setting all properties.
function KD_CreateFcn(hObject, eventdata, handles)
% hObject    handle to KD (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end



function trajectory_Callback(hObject, eventdata, handles)
% hObject    handle to trajectory (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of trajectory as text
%        str2double(get(hObject,'String')) returns contents of trajectory as a double

trajectory = get(hObject,'String');
ii = 1;
while ii <= numel(trajectory)
    str = trajectory{ii};
    if ~isempty(str)
        if strcmpi(trajectory{ii}(1:3),'for')
            jj = ii;
            while ~strcmpi(trajectory{ii},'end')
                ii = ii+1;
            end
            for kk = jj+1:ii
                str = [str '; ' trajectory{kk}];
            end
        end
        eval([str ';'])
    end
    ii = ii+1;
end

plot(t,theta);
handles.traj = theta;
% handles.PWM = PWM;
xlabel('Time (s)'); ylabel('\theta (\circ)');
set(handles.trajectory,'String',trajectory);

% Update handles structure
guidata(hObject, handles);


% --- Executes during object creation, after setting all properties.
function trajectory_CreateFcn(hObject, eventdata, handles)
% hObject    handle to trajectory (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end
